##################################################### #
#         Monte Carlo Experiment (LC-MNL) with 3 Classes
# This script run the monte carlo experiment for Latent Class Models (3 classes)
# By: Mauricio Sarrias
# Date: 22 March, 2016
# Last Modification: September 27, 2017 (For review)
###################################################### #

# Common setting for all the estimations ====
rm(list = ls(all = TRUE))
library("gmnl")
library("mlogit")
source('R Script/boot.lc.R')

my.time <- function(object){
  et <- object[3]
  s <- round(et,0)
  h <- s %/% 3600
  s <- s - 3600*h
  m <- s %/% 60
  s <- s - 60*m
  paste(h, "h:", m, "m:", s, "s", sep = "")
}

# Set seed
set.seed(1234)

#=======================================================#
# Experiment 3: N = 1000, Boot = 10000 , Classes = 3 ----
#=======================================================#

## Globals
N <- 1000                     # Number of individuals as Revelt & Train (2000)
J <- 3                        # Number of alternatives as Revelt & Train (2000)
times <- c(1, 5, 10, 20, 50)  # Choice situation per individual as Revelt & Train (2000)
S <- 300                      # Number of databases
Q <- 1000                     # Number of Boot for Krisky Robb procedure.


## Generate True Random Coefficients
b1_e3 <- c(rep(-2,   N * 0.25),   rep(0,   N * 0.5), rep(2,   N * 0.25))
b2_e3 <- c(rep(-0.5, N * 0.25),   rep(1,   N * 0.5), rep(1.5, N * 0.25))

true_class_e3 <- c(rep(1,   N * 0.25),   rep(2,   N * 0.5),  rep(3,   N * 0.25))


## Matrix for results b11 class 1 var 1, b12 class 1, var2
b11.sim.e3  <- matrix(NA, nrow = S, ncol = length(times))
b12.sim.e3  <- matrix(NA, nrow = S, ncol = length(times))
b21.sim.e3  <- matrix(NA, nrow = S, ncol = length(times))
b22.sim.e3  <- matrix(NA, nrow = S, ncol = length(times))
b31.sim.e3  <- matrix(NA, nrow = S, ncol = length(times))
b32.sim.e3  <- matrix(NA, nrow = S, ncol = length(times))
con2.sim.e3 <- matrix(NA, nrow = S, ncol = length(times))
con3.sim.e3 <- matrix(NA, nrow = S, ncol = length(times))
share1.e3 <- matrix(NA, nrow = S, ncol = length(times))
share2.e3 <- matrix(NA, nrow = S, ncol = length(times))
share3.e3 <- matrix(NA, nrow = S, ncol = length(times))


clas.as3 <- clasw.as3 <- array(NA, dim = c(S, N, ncol = length(times)))

bi1.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))
bi2.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))

si1.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))
si2.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))

bi1b.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))
bi2b.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))

si1b.sim.e3 <- li1b.sim.e3 <- ui1b.sim.e3  <- array(NA, dim = c(S, N, ncol = length(times)))
si2b.sim.e3 <- li2b.sim.e3 <- ui2b.sim.e3  <- array(NA, dim = c(S, N, ncol = length(times)))


bi1w.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))
bi2w.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))

si1w.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))
si2w.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))

bi1bw.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))
bi2bw.sim.e3 <- array(NA, dim = c(S, N, ncol = length(times)))

si1bw.sim.e3 <- li1bw.sim.e3 <- ui1bw.sim.e3  <- array(NA, dim = c(S, N, ncol = length(times)))
si2bw.sim.e3 <- li2bw.sim.e3 <- ui2bw.sim.e3  <- array(NA, dim = c(S, N, ncol = length(times)))


mix.time.e3  <- matrix(NA, nrow = S, ncol = length(times))
boot.time.e3 <- matrix(NA, nrow = S, ncol = length(times))
mess.e3      <- matrix(NA, nrow = S, ncol = length(times))


## Monte Carlo Loop
for (c in 1:length(times)) {
  # Choice situation
  t <- times[c]
  # id for individuals
  id <- rep(1:N, each = J * t)
  
  for (s in 1:S) {
    cat("\n","===========================", "\n")
    cat("Time: ", t, ", and database: ", s, "\n")
    cat("================================", "\n")
    
    # Generate variables that depend on T
    x1 <- rnorm(N * J * t)
    x2 <- as.numeric(runif(N * J * t) < 0.5)
    
    # The true data generating process
    U <-  b1_e3[id] * x1 + b2_e3[id] * x2 - log(-log(runif(N * J * t))) 
    
    ## Chosen alternative
    chosen <- rep(0, N * J * t)
    for (i in 1:(N * t)) {
      U_j <- U[((i - 1) * J + 1):(i * J)]
      U_max <- max(U_j)
      pos <- which(U_j == U_max)
      chosen[((i - 1) * J + 1):(i * J)][pos] <- TRUE
    }
    
    ## Data format
    data <- as.data.frame(cbind(id, chosen, x1, x2))
    data <- mlogit.data(data, choice = "chosen", shape = "long", 
                        alt.levels = c("1", "2", "3"), id.var = "id")
    # Estimate LC-MNL
    stm <- proc.time()
    if (t == 1) {
      # True model
      lc <- gmnl(chosen ~ x1 + x2 | 0 | 0 | 0 | 1, 
                 data = data,
                 model = "lc",
                 print.init = TRUE,
                 Q = 3,  
                 seed = NULL,
                 iterlim = 1000)
      # Misspecified Model
      lcw <- gmnl(chosen ~ x1 + x2 | 0 | 0 | 0 | 1, 
                 data = data,
                 model = "lc",
                 print.init = TRUE,
                 Q = 2,  
                 seed = NULL,
                 iterlim = 1000)
    } else {
      # True model
      lc <- gmnl(chosen ~ x1 + x2  | 0 | 0 | 0 | 1, 
                 data = data,
                 model = "lc",
                 print.init = TRUE,
                 Q = 3,  
                 seed = NULL,
                 iterlim = 1000,
                 panel = TRUE)
      # Misspecified Model
      lcw <- gmnl(chosen ~ x1 + x2  | 0 | 0 | 0 | 1, 
                 data = data,
                 model = "lc",
                 print.init = TRUE,
                 Q = 2,  
                 seed = NULL,
                 iterlim = 1000,
                 panel = TRUE)
    }
    mix.time.e3[s, c] <- (proc.time() - stm)[3]
    cat("\n", "Estimation took: ", my.time(proc.time() - stm), "\n")
    
    cat("\n", "Coefficients are : ", "\n")
    print(coef(lc))
    mess.e3[s, c] <- lc$logLik$message
    
    cat("\n", "How is my LL doing?: ", mess.e3[s, c], "\n")
    b11.sim.e3[s, c] <- coef(lc)[1]
    b12.sim.e3[s, c] <- coef(lc)[2]
    b21.sim.e3[s, c] <- coef(lc)[3]
    b22.sim.e3[s, c] <- coef(lc)[4]
    b31.sim.e3[s, c] <- coef(lc)[5]
    b32.sim.e3[s, c] <- coef(lc)[6]
    
    con2.sim.e3[s, c] <- coef(lc)[7]
    con3.sim.e3[s, c] <- coef(lc)[8]
    
    
    ## Compute probability for each class
    share1.e3[s, c] <- exp(0) / (exp(0) + exp(con2.sim.e3[s, c]) + exp(con3.sim.e3[s, c]))
    share2.e3[s, c] <- exp(con2.sim.e3[s, c]) / (exp(0) + exp(con2.sim.e3[s, c]) + exp(con3.sim.e3[s, c]))
    share3.e3[s, c] <- exp(con3.sim.e3[s, c]) / (exp(0) + exp(con2.sim.e3[s, c]) + exp(con3.sim.e3[s, c]))
    
    ## Compute individual-estimates
    bq  <- lc$bi ;  bqw <- lcw$bi
    Qiq <- lc$Qir; Qiqw <- lcw$Qir
    N   <- nrow(Qiq)
    K   <- ncol(bq); Kw <- ncol(bqw)
    
    mean <- meanw <-  matrix(NA, N, K)
    se   <- sew   <- matrix(NA, N, K)
    colnames(mean) <- colnames(meanw) <- colnames(sew) <- colnames(se) <- colnames(bq)
    
    for (j in 1:K) {
      b_temp  <- gmnl:::repRows(bq[, j], N) 
      b_tempw <- gmnl:::repRows(bqw[, j], N)  
      mean[, j]  <- rowSums(b_temp * Qiq)
      meanw[, j] <- rowSums(b_tempw * Qiqw) 
      se[, j]   <- sqrt(rowSums((b_temp - mean[, j])^2 * Qiq))
      sew[, j]   <- sqrt(rowSums((b_tempw - meanw[, j])^2 * Qiqw))
    }
    
    clas.as3[s, ,c]  <- apply(Qiq, 1, which.max)
    clasw.as3[s, ,c] <- apply(Qiqw, 1, which.max)
    
    bi1.sim.e3[s, , c] <- mean[, "x1"]
    bi2.sim.e3[s, , c] <- mean[, "x2"]
    si1.sim.e3[s, , c] <- se[, "x1"]
    si2.sim.e3[s, , c] <- se[, "x2"]
    
    bi1w.sim.e3[s, , c] <- meanw[, "x1"]
    bi2w.sim.e3[s, , c] <- meanw[, "x2"]
    si1w.sim.e3[s, , c] <- sew[, "x1"]
    si2w.sim.e3[s, , c] <- sew[, "x2"]
    
    # Bootstrapping
    sbt <- proc.time()
    tryCatch({
      if (t == 1) {
        bootS <- boot.lc(lc, panel =  FALSE, S = Q, Q = 3) 
        bootSw <- boot.lc(lcw, panel =  FALSE, S = Q, Q = 2) 
      } else {
        bootS  <- boot.lc(lc, panel =  TRUE,  S = Q, Q = 3) 
        bootSw <- boot.lc(lcw, panel =  TRUE, S = Q, Q = 2) 
      } 
      bi1b.sim.e3[s, , c] <- bootS$mean[ ,"x1"]
      bi2b.sim.e3[s, , c] <- bootS$mean[ ,"x2"]
      si1b.sim.e3[s, , c] <- bootS$se[ ,"x1"]
      si2b.sim.e3[s, , c] <- bootS$se[ ,"x2"]
      li1b.sim.e3[s, , c] <- bootS$quant[1, ,"x1"]
      ui1b.sim.e3[s, , c] <- bootS$quant[2, ,"x1"]
      li2b.sim.e3[s, , c] <- bootS$quant[1, ,"x2"]
      ui2b.sim.e3[s, , c] <- bootS$quant[2, ,"x2"]
      
      bi1bw.sim.e3[s, , c] <- bootSw$mean[ ,"x1"]
      bi2bw.sim.e3[s, , c] <- bootSw$mean[ ,"x2"]
      si1bw.sim.e3[s, , c] <- bootSw$se[ ,"x1"]
      si2bw.sim.e3[s, , c] <- bootSw$se[ ,"x2"]
      li1bw.sim.e3[s, , c] <- bootSw$quant[1, ,"x1"]
      ui1bw.sim.e3[s, , c] <- bootSw$quant[2, ,"x1"]
      li2bw.sim.e3[s, , c] <- bootSw$quant[1, ,"x2"]
      ui2bw.sim.e3[s, , c] <- bootSw$quant[2, ,"x2"]
      
      cat("\n", "Bootstrapping took: ", my.time(proc.time() - sbt), "\n")
      boot.time.e3[s, c] <- (proc.time() - sbt)[3]
      cat("\n", "Mean boot for ce x1: ", mean(bootS$mean[ ,"x1"]), "\n")
      cat("\n", "Mean boot for se of ce x1: ", mean(bootS$se[ ,"x1"]), "\n")
    }, error = function(e) {cat("ERROR :", conditionMessage(e), "\n")}
    )
  }
}


save.image(file = "Data/LC_experimentAllMiss.RData")

